{
ZZZZZZ 5.0, a chess program.
Copyright (C) 1998 G. Wiesenekker
E-mail: gijsbert.wiesenekker@wxs.nl
http://home.wxs.nl/~gijsbert.wiesenekker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit Cache;

{$MODE Delphi}

interface

uses Chess;

procedure SetCacheLength;
procedure SetQuiesCacheLength;
procedure SetScoreCacheLength;
procedure ClearCache;
procedure ClearQuiesCache;
procedure ClearScoreCache;
procedure ResetCacheDepth;

function ReturnCacheIndex: Integer;
function ReturnQuiesCacheIndex: Integer;
function ReturnScoreCacheIndex: Integer;

implementation

const
	KiloByte = 1024;
	MegaByte = KiloByte * KiloByte;
var
	CacheLength, QuiesCacheLength, ScoreCacheLength: Integer;

function IsPrime(N: Integer): Boolean;
var
	i: Integer;
begin
	Result := False;
	if not Odd(N) then Exit;
  i := 3;
  while (i * i) <= N do
  begin
  	if (N Mod i) = 0 then Exit;
  	Inc(i, 2);
  end;
  Result := True;
end;

procedure MakePrime(var N: Integer);
begin
	if not Odd(N) then
  	Dec(N);
	while not IsPrime(N) do
    Dec(N, 2);
end;

procedure SetCacheLength;
begin
  CacheLength := CacheSize * Megabyte div SizeOf(CacheRecord) div 2;
  MakePrime(CacheLength);
  SetLength(AlphaBetaCache[White], CacheLength);
  SetLength(AlphaBetaCache[Black], CacheLength);
end;

procedure SetQuiesCacheLength;
begin
  QuiesCacheLength := QuiesCacheSize * Megabyte div SizeOf(QuiesCacheRecord) div 2;
  MakePrime(QuiesCacheLength);
  SetLength(QuiesCache[White], QuiesCacheLength);
  SetLength(QuiesCache[Black], QuiesCacheLength);
end;

procedure SetScoreCacheLength;
begin
  ScoreCacheLength := ScoreCacheSize * Megabyte div SizeOf(ScoreCacheRecord) div 2;
  MakePrime(ScoreCacheLength);
  SetLength(ScoreCache[White], ScoreCacheLength);
  SetLength(ScoreCache[Black], ScoreCacheLength);
end;

procedure ClearCache;
var
	i: Integer;
begin
	for i := 0 to CacheLength - 1 do
  begin
  	AlphaBetaCache[White][i].CacheKey := 0;
  	AlphaBetaCache[Black][i].CacheKey := 0;
  end;
end;

procedure ClearQuiesCache;
var
	i: Integer;
begin
	for i := 0 to QuiesCacheLength - 1 do
  begin
  	QuiesCache[White][i].QuiesCacheKey := 0;
  	QuiesCache[Black][i].QuiesCacheKey := 0;
  end;
end;

procedure ClearScoreCache;
var
	i: Integer;
begin
	for i := 0 to ScoreCacheLength - 1 do
  begin
  	ScoreCache[White][i].ScoreCacheKey := 0;
  	ScoreCache[Black][i].ScoreCacheKey := 0;
  end;
end;

function ReturnCacheIndex;
begin
	Result := NodeList[Node].NodeFullHashKey mod CacheLength;
end;

function ReturnQuiesCacheIndex;
begin
	Result := NodeList[Node].NodeFullHashKey mod QuiesCacheLength;
end;

function ReturnScoreCacheIndex;
begin
	Result := NodeList[Node].NodeFullHashKey mod ScoreCacheLength;
end;

procedure ResetCacheDepth;
var
	i: Integer;
begin
	for i := 0 to CacheLength - 1 do
  begin
  	AlphaBetaCache[White][i].CacheDepth := -1;
  	AlphaBetaCache[Black][i].CacheDepth := -1;
  end;
end;

end.
