{
ZZZZZZ 5.0, a chess program.
Copyright (C) 1998 G. Wiesenekker
E-mail: gijsbert.wiesenekker@wxs.nl
http://home.wxs.nl/~gijsbert.wiesenekker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit Keys;

{$MODE Delphi}

interface

uses Chess, Rand;

function HashKeyFromBoard: Key;
function HashKeyFromLists: Key;
procedure CheckHashKey;
procedure SetFullHashKey;

implementation

uses SysUtils, Lists;

function HashKeyFromBoard: Key;
var
	iBoard: BoardRange;
begin
	Result := 0;
  for iBoard := Low(BoardRange) to High(BoardRange) do
  begin
  	with Board[iBoard]^ do
    begin
    	if (ListColour <> None) then
    		Result := Result xor ListKey[iBoard];
    end;
  end;
end;

function HashKeyFromLists: Key;
var
	Colour: BoardColourEnum;
  Kind: BoardKindEnum;
  i: ExtBoardRange;
begin
	Result := 0;
  for Colour := Low(BoardColourEnum) to High(BoardColourEnum) do
  begin
  	for Kind := Low(BoardKindEnum) to High(BoardKindEnum) do
    begin
    	for i := 1 to PieceCount[Colour, Kind] do
      begin
      	with PieceList[Colour, Kind, i] do
        begin
        	if ListWhere = Low(ExtBoardRange) then
        		continue;
        	Result := Result xor ListKey[ListWhere];
      	end;
    	end;
    end;
  end;
end;

procedure CheckHashKey;
var
	Temp1, Temp2: Key;
begin
  Temp1 := HashKeyFromBoard;
  Temp2 := HashKeyFromLists;
  if (Temp1 <> Temp2) then
  	raise EChessError.Create('Key error. Key from board: ' + IntToStr(Temp1) +
    	' Key from lists: ' + IntToStr(Temp2));
  if (Temp1 <> HashKey) then
  	raise EChessError.Create('Key error. HashKey: ' + IntToStr(HashKey) +
    	' Key from lists/board: ' + IntToStr(Temp1));
end;

procedure SetFullHashKey;
var
	TempKey: Key;
begin
	TempKey := HashKey;
  if not Board[E1].ListMoved then
  begin
  	if not Board[H1].ListMoved then
    	TempKey := TempKey xor WhiteShortCastleKey;
  	if not Board[A1].ListMoved then
    	TempKey := TempKey xor WhiteLongCastleKey;
  end;
  if not Board[E8].ListMoved then
  begin
  	if not Board[H8].ListMoved then
    	TempKey := TempKey xor BlackShortCastleKey;
  	if not Board[A8].ListMoved then
    	TempKey := TempKey xor BlackLongCastleKey;
  end;
  if NodeList[Node].NodeEnPassant <> Low(ExtBoardRange) then
  	TempKey := TempKey xor EnPassantKey[NodeList[Node].NodeEnPassant];
	NodeList[Node].NodeFullHashKey := TempKey;
end;

end.
