{
ZZZZZZ 5.0, a chess program.
Copyright (C) 1998 G. Wiesenekker
E-mail: gijsbert.wiesenekker@wxs.nl
http://home.wxs.nl/~gijsbert.wiesenekker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit Main;

{$MODE Delphi}

interface
uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Menus, Chess;

type
  TMainForm = class(TForm)
    BoardGrid: TStringGrid;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    NewItem: TMenuItem;
    ExitItem: TMenuItem;
    MovesGrid: TStringGrid;
    GameMovesGrid: TStringGrid;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    SaveItem: TMenuItem;
    LoadItem: TMenuItem;
    OutputMemo: TMemo;
    DebugMenu: TMenuItem;
    ShowRandomItem: TMenuItem;
    EditMenu: TMenuItem;
    SetupItem: TMenuItem;
    PreferencesItem: TMenuItem;
    ShowListsItem: TMenuItem;
    ParametersItem: TMenuItem;
    GameMovesPopupMenu: TPopupMenu;
    ClearMovesItem: TMenuItem;
    HelpMenu: TMenuItem;
    AboutItem: TMenuItem;
    SearchMenu: TMenuItem;
    ShowVisitItem: TMenuItem;
    ShowScoreItem: TMenuItem;
    SettingsItem: TMenuItem;
    ViewMenu: TMenuItem;
    InvertBoardItem: TMenuItem;
    StartSearchButton: TButton;
    AbortSearchButton: TButton;
    NodesEdit: TEdit;
    NodesLabel: TLabel;
    TimeUsedEdit: TMenuItem;
    BrowseButton: TButton;
    Markregistryold1: TMenuItem;
    procedure BoardGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure NewItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure BoardGridMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BoardGridDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure BoardGridDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure SaveItemClick(Sender: TObject);
    procedure LoadItemClick(Sender: TObject);
    procedure ShowRandomItemClick(Sender: TObject);
    procedure SetupItemClick(Sender: TObject);
    procedure PreferencesItemClick(Sender: TObject);
    procedure InvertBoardItemClick(Sender: TObject);
    procedure ShowListsItemClick(Sender: TObject);
    procedure MovesGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure GameMovesGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure MovesGridClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ParametersItemClick(Sender: TObject);
    procedure ClearMovesItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure ShowVisitItemClick(Sender: TObject);
    procedure ShowScoreItemClick(Sender: TObject);
    procedure SettingsItemClick(Sender: TObject);
    procedure StartSearchButtonClick(Sender: TObject);
    procedure AbortSearchButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TimeUsedEditClick(Sender: TObject);
    procedure GameMovesGridClick(Sender: TObject);
    procedure BrowseButtonClick(Sender: TObject);
    procedure Markregistryold1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
var
  MainForm: TMainForm;
procedure UpdateWindows;
procedure DrawBoard;
procedure HandleMoveInput(TheMove: AnsiString);

implementation

uses Utils, DebugRand, Setup, Prefs, Lists, DebugLists, Moves, Cache, Score,
	Parameters, Search, About, DebugVisit, Thread, Settings, DebugScore,
  TimeUsed;

{$R *.lfm}

var
	DragCol, DragRow: Integer;
  NextGameCol, NextGameRow: Integer;
  ProcessingMessages: Boolean;

procedure StopSearch;
begin
  ChangeSearchStatus(SearchStopping);
  ProcessingMessages := True;
	while SearchStatus <> SearchStopped do
  	Application.ProcessMessages;
  ProcessingMessages := False;
end;

procedure AbortSearch;
begin
  ChangeSearchStatus(SearchAborting);
  ProcessingMessages := True;
	while SearchStatus <> SearchStopped do
  	Application.ProcessMessages;
  ProcessingMessages := False;
end;

procedure UpdateWindows;
begin
  ProcessingMessages := True;
	Application.ProcessMessages;
  ProcessingMessages := False;
end;

procedure TMainForm.BoardGridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
	StringGridDrawCell(BoardGrid, ACol, ARow, Rect, State);
end;

procedure DrawBoard;
var
	Col, Row: Integer;
  DrawCol, DrawRow: Integer;
  Map: AnsiString;
begin
	for Col := 0 to 7 do
  begin
  	for Row := 0 to 7 do
    begin
    	with Board[RowColToBoardIndex[Row, Col]]^ do
      begin
      	if ListColour = White then
        begin
        	if ListKind = Pawn then
          	Map := wP
        	else if ListKind = Knight then
          	Map := wN
        	else if ListKind = Bishop then
          	Map := wB
        	else if ListKind = Rook then
          	Map := wR
        	else if ListKind = Queen then
          	Map := wQ
        	else if ListKind = King then
          	Map := wK
          else
          	raise EChessError.Create('ListKind Error');
        end
        else if ListColour = Black then
        begin
        	if ListKind = Pawn then
          	Map := bP
        	else if ListKind = Knight then
          	Map := bN
        	else if ListKind = Bishop then
          	Map := bB
        	else if ListKind = Rook then
          	Map := bR
        	else if ListKind = Queen then
          	Map := bQ
        	else if ListKind = King then
          	Map := bK
          else
          	raise EChessError.Create('ListKind Error');
        end
        else
        	Map := nE;
        if ListCaptured then
        	Map := Map + '*';
        if MainForm.InvertBoardItem.Checked then
        begin
        	DrawCol := 7 - Col;
          DrawRow := 7 - Row;
        end
        else
        begin
        	DrawCol := Col;
          DrawRow := Row;
        end;
        if Map <> MainForm.BoardGrid.Cells[DrawCol, DrawRow] then
        	MainForm.BoardGrid.Cells[DrawCol, DrawRow] := Map;
    	end;
    end;
  end;
  UpdateWindows;
end;

procedure DrawMoves;
var
  iMove: ExtMoveRange;
begin
  ClearStringGrid(MainForm.MovesGrid);
  for iMove := 1 to nThreadMoves do
  begin
  	MainForm.MovesGrid.Cells[0, iMove] := MoveToLongString(ThreadMoves[iMove]);
  	MainForm.MovesGrid.Cells[1, iMove] := MoveToDragDropString(ThreadMoves[iMove]);
  end;
  MainForm.MovesGrid.OnClick := nil;
  MainForm.MovesGrid.Row := 1;
  MainForm.MovesGrid.Col := 0;
  MainForm.MovesGrid.OnClick := MainForm.MovesGridClick;
  UpdateWindows;
end;

function GoodPosition(MyColour, YourColour: BoardColourEnum): Boolean;
var
	MyCheck, YourCheck: CheckEnum;
  nMoves: ExtMoveRange;
  Moves: MoveList;
  procedure ShowResult(Result: AnsiString);
  begin
		Application.MessageBox(PChar(Result), 'Confirm', MB_OK);
  end;
label
	ShowError;
begin
	Result := False;
  if (PieceCount[White, King] <> 1) then
  begin
  	ShowResult('White should have at least one king!');
    Exit;
  end;
  if (PieceCount[Black, King] <> 1) then
  begin
  	ShowResult('Black should have at least one king!');
    Exit;
  end;
  MyCheck := ReturnCheck(MyColour, YourColour);
  YourCheck := ReturnCheck(YourColour, MyColour);
  if (MyCheck = Check) and (YourCheck = Check) then
  begin
  	ShowResult('Both kings are in check!');
    Exit;
  end;
  GenerateLegalMoves(MyColour, YourColour, nMoves, Moves);
  if nMoves = 0 then
  begin
  	if MyCheck = Check then
  		ShowResult('Checkmate!')
    else
  		ShowResult('Stalemate!');
    Exit;
  end;
  if DrawByRepetition(True) then
  begin
  	ShowResult('Draw by repetition!');
    Exit;
  end;
  Result := True;
end;

procedure ReadGameMoves;

	procedure AllocateTime;
  var
  	NextMove, MovesToGo: Integer;
    TimeLeft: Integer;
  begin
    NextMove := NextGameRow;
    if Options.TimeLimit > 0 then
    begin
    	TimeSearch := Options.TimeLimit;
      TimeProblem := TimeSearch;
    end
    else
    begin
      if (NextMove mod 5) = 4 then
      begin
        TimeUsedForm.ShowModal;
      end;
      if NextMove <= GameMovesFirstPeriod then
      begin
        MovesToGo := GameMovesFirstPeriod - NextMove + 1;
        TimeLeft := GameTimeFirstPeriod * 60 - GameTimeUsed;
        TimeSearch := TimeLeft div MovesToGo;
        TimeProblem := TimeSearch;
      end
      else
      begin
        MovesToGo := GameMovesSecondPeriod;
        TimeLeft := (GameTimeFirstPeriod + GameTimeSecondPeriod) * 60 - GameTimeUsed;
        TimeSearch := TimeLeft div MovesToGo;
        TimeProblem := TimeSearch;
      end;
      if TimeSearch < 5 then
      begin
        TimeSearch := 5;
        TimeProblem := TimeSearch;
      end;
    end;
    MainForm.OutputMemo.Lines.Add('allocated ' + IntToStr(TimeSearch));
  end;
var
	Row, Col, FromCol, ToCol: Integer;
  iMove: ExtMoveRange;
  Append: Boolean;
  Found: Boolean;
begin
	SetLists;
  if GameColour = White then
  begin
  	if not GoodPosition(White, Black) then
    	Exit;
  end
  else
  begin
  	if not GoodPosition(Black, White) then
    	Exit;
  end;
  for Row := 1 to NextGameRow do
  begin
  	if Row = 1 then
    begin
    	if GameColour = White then
      	FromCol := 1
      else
      	FromCol := 2;
    end
    else
    begin
			FromCol := 1;
    end;
    if Row < NextGameRow then
      ToCol := 2
    else
      ToCol := NextGameCol;
  	for Col := FromCol to ToCol do
    begin
      if Col = 1 then
      begin
        MyThreadColour := White;
        YourThreadColour := Black;
      end
      else
      begin
        MyThreadColour := Black;
        YourThreadColour := White;
      end;
      GenerateLegalMoves(MyThreadColour, YourThreadColour, nThreadMoves, ThreadMoves);
      if (Col <> NextGameCol) or (Row <> NextGameRow) then
      begin
      	Found := False;
        for iMove := 1 to nThreadMoves do
        begin
          if MoveToLongString(ThreadMoves[iMove]) = MainForm.GameMovesGrid.Cells[Col, Row] then
          begin
            DoMove(MyThreadColour, YourThreadColour, ThreadMoves[iMove]);
            Found := True;
            break;
          end;
        end;
        if not Found then
        	raise EChessError.Create('Move not found');
      end;
    end;
  end;
	DrawBoard;
  if not GoodPosition(MyThreadColour, YourThreadColour) then
    Exit;
  DrawMoves;
  CompactLists;
  if MainForm.OutputMemo.Lines.Count > 100 then
  	MainForm.OutputMemo.Lines.Clear;
  DisplayScore;
  Append := False;
  if GameMode = HumanHuman then
  begin
  	Append := False;
  end
  else if GameMode = HumanComputer then
  begin
  	if MyThreadColour = White then
	  	Append := False
    else
  		Append := True;
  end
  else if GameMode = ComputerHuman then
  begin
  	if MyThreadColour = Black then
	  	Append := False
    else
  		Append := True;
  end
  else if GameMode = ComputerComputer then
  begin
		Append := True;
  end;
  if GameMode <> NoSearch then
  begin
  	if Append then
  	begin
    	AllocateTime;
    	SearchStatus := AppendStarting;
  	end
  	else
  	begin
    	TimeSearch := 999999;
      TimeProblem := 0;
    	SearchStatus := SearchStarting;
  	end
  end;
end;

procedure TMainForm.NewItemClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	GameBoardGrid := StartBoardGrid;
  CastlingRights := [WhiteShort, WhiteLong, BlackShort, BlackLong];
  GameColour := White;
  ClearStringGrid(MovesGrid);
  ClearStringGrid(GameMovesGrid);
  if GameColour = White then
  	NextGameCol := 1
  else
  	NextGameCol := 2;
  NextGameRow := 1;
  ReadGameMoves;
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	if Application.MessageBox('Ok to exit?', 'Confirm', MB_OKCANCEL) = IDOK then
	begin
		AbortSearch;
		Application.Terminate;
  end;
end;

procedure TMainForm.MovesGridClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
  if (Sender <> nil) then
		AbortSearch;
  GameMovesGrid.Cells[NextGameCol, NextGameRow] :=
  	MovesGrid.Cells[MovesGrid.Col, MovesGrid.Row];
  Inc(NextGameCol);
  if NextGameCol > 2 then
  begin
  	Inc(NextGameRow);
    NextGameCol := 1;
  end;
  GameMovesGrid.OnClick := nil;
  GameMovesGrid.Row := NextGameRow;
  GameMovesGrid.Col := NextGameCol;
  GameMovesGrid.OnClick := GameMovesGridClick;
  ReadGameMoves;
end;

procedure TMainForm.BoardGridMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if ProcessingMessages then
  	Exit;
	BoardGrid.MouseToCell(X, Y, DragCol, DragRow);
  if MainForm.InvertBoardItem.Checked then
  begin
    DragCol := 7 - DragCol;
    DragRow := 7 - DragRow;
  end;
	BoardGrid.BeginDrag(True);
end;

procedure TMainForm.BoardGridDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
	//
end;

procedure HandleMoveInput(TheMove: AnsiString);
var
	Move: Integer;
begin
  Move := 1;
  while Move < MainForm.MovesGrid.RowCount do
  begin
  	if MainForm.MovesGrid.Cells[0, Move] = '' then
    	break;
  	if MainForm.MovesGrid.Cells[0, Move] = TheMove then
		begin
      if Options.UseSounds then
      begin
      	Beep;
      end;
	  	MainForm.OutputMemo.Lines.Add(TheMove);
      MainForm.MovesGrid.OnClick := nil;
    	MainForm.MovesGrid.Col := 0;
    	MainForm.MovesGrid.Row := Move;
      MainForm.MovesGrid.OnClick := MainForm.MovesGridClick;
			MainForm.MovesGridClick(nil);
    	break;
    end;
    Inc(Move);
  end;
end;

procedure TMainForm.BoardGridDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
	Col, Row: Integer;
  DragDropMove: AnsiString;
	Move: Integer;
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	BoardGrid.MouseToCell(X, Y, Col, Row);
  if MainForm.InvertBoardItem.Checked then
  begin
    Col := 7 - Col;
    Row := 7 - Row;
  end;
  DragDropMove := BoardIndexToString(RowColToBoardIndex[DragRow, DragCol]) +
  	BoardIndexToString(RowColToBoardIndex[Row, Col]);
  Move := 1;
  while Move < MovesGrid.RowCount do
  begin
  	if MovesGrid.Cells[1, Move] = '' then
    	break;
  	if MovesGrid.Cells[1, Move] = DragDropMove then
		begin
	  	OutputMemo.Lines.Add(DragDropMove);
      MovesGrid.OnClick := nil;
    	MovesGrid.Col := 0;
    	MovesGrid.Row := Move;
      MovesGrid.OnClick := MovesGridClick;
			MovesGridClick(nil);
    	break;
    end;
    Inc(Move);
  end;
end;

procedure TMainForm.SaveItemClick(Sender: TObject);
var
	TheFile: TextFile;
  Row, Col: Integer;
begin
	SaveDialog.FileName := 'Untitled.zzz';
	if (SaveDialog.Execute) then
  begin
		AssignFile(TheFile, SaveDialog.FileName);
    Rewrite(TheFile);
    //save board
    for Row := 0 to 7 do
    begin
    	for Col := 0 to 7 do
      	writeln(TheFile, GameBoardGrid[Row, Col]);
    end;
    if GameColour = White then
  		writeln(TheFile, 'white')
    else
    	writeln(TheFile, 'black');
    //save moves
    for Row := 1 to NextGameRow - 1 do
    begin
    	for Col := 1 to 2 do
    		writeln(TheFile, GameMovesGrid.Cells[Col, Row]);
    end;
    for Col := 1 to NextGameCol - 1 do
			writeln(TheFile, GameMovesGrid.Cells[Col, NextGameRow]);
    CloseFile(TheFile);
  end;
end;

procedure TMainForm.LoadItemClick(Sender: TObject);
var
	TheFile: TextFile;
  Row, Col: Integer;
	TheString: AnsiString;
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	//TheString := ',,1';
  //BoardGrid.Rows[0].CommaText := TheString;
  //TheString := BoardGrid.Cells[0,0];
	if (OpenDialog.Execute) then
  begin
		AssignFile(TheFile, OpenDialog.FileName);
    Reset(TheFile);
    //load board
    for Row := 0 to 7 do
    begin
    	for Col := 0 to 7 do
      	readln(TheFile, GameBoardGrid[Row, Col]);
    end;
    readln(TheFile, TheString);
    if TheString = 'white' then
    	GameColour := White
    else
    	GameColour := Black;
    //load moves
    NextGameRow := 1;
    NextGameCol := 1;
    while (not eof(TheFile)) do
    begin
    	readln(TheFile, TheString);
    	GameMovesGrid.Cells[NextGameCol, NextGameRow] := TheString;
      Inc(NextGameCol);
      if NextGameCol > 2 then
      begin
      	Inc(NextGameRow);
        NextGameCol := 1;
      end;
    end;
    CloseFile(TheFile);
		ReadGameMoves;
  end;
end;

procedure TMainForm.ShowRandomItemClick(Sender: TObject);
begin
	DebugRandForm.ShowModal;
end;

procedure TMainForm.SetupItemClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	if SetupForm.ShowModal = mrOk then
  begin
    ClearStringGrid(MovesGrid);
    ClearStringGrid(GameMovesGrid);
    if GameColour = White then
      NextGameCol := 1
    else
      NextGameCol := 2;
    NextGameRow := 1;
		ReadGameMoves;
  end;
end;

procedure TMainForm.PreferencesItemClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	PrefsForm.ShowModal();
  ClearCache;
  ClearScoreCache;
end;

procedure TMainForm.InvertBoardItemClick(Sender: TObject);
var
	Row, Col: Integer;
  TempGrid: BoardGridType;
begin
	InvertBoardItem.Checked := not InvertBoardItem.Checked;
  for Col := 0 to 7 do
  begin
  	for Row := 0 to 7 do
    begin
    	TempGrid[Row, Col] := BoardGrid.Cells[Col, Row];
    end;
  end;
  for Col := 0 to 7 do
  begin
  	for Row := 0 to 7 do
    begin
      BoardGrid.Cells[7 - Col, 7 - Row] := TempGrid[Row, Col];
    end;
  end;
end;

procedure TMainForm.ShowListsItemClick(Sender: TObject);
begin
	//SetLists;
  DebugListsForm.ShowModal;
end;

procedure TMainForm.MovesGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
	CanSelect := (MovesGrid.Cells[ACol, ARow] <> '');
end;

procedure TMainForm.GameMovesGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
	//CanSelect := (GameMovesGrid.Cells[ACol, ARow] <> '');
  CanSelect := True;
end;

procedure TMainForm.FormShow(Sender: TObject);
  function FontInstalled(Name: AnsiString): Boolean;
	var
  	iFont: Integer;
  begin
  	Result := True;
    for iFont := 0 to  Screen.Fonts.Count - 1 do
    begin
      if Screen.Fonts.Strings[iFont] = Name then
				Exit;
    end;
    Result := False;
  end;
begin;
  ProcessingMessages := False;
  AboutForm.ShowModal;
  //check font
  if not FontInstalled('Chess Merida') then
  begin
    Application.MessageBox('Please install the Chess Merida Font', 'Error', MB_OK);
    Application.Terminate;
  end;
  UpgradeRegistry(2);
  FontSize := StrToInt(ReadRegValString(RegValFontSize, '32'));
  Debug := StrToBool(ReadRegValString(RegValDebug, 'False'));
  Options.UseCache := True;
  Options.UseNullMove := True;
  Options.UseLazy := True;
  Options.ExtendCheck := False;
  Options.UseSounds := True;
  //
  CacheSize := StrToInt(ReadRegValString(RegValCacheSize, '2'));
  QuiesCacheSize := StrToInt(ReadRegValString(RegValQuiesCacheSize, '2'));
  ScoreCacheSize := StrToInt(ReadRegValString(RegValScoreCacheSize, '4'));
  //
  MaterialValue[Pawn] := StrToInt(ReadRegValString(RegValPawnValue, '100'));
  MaterialValue[Knight] := StrToInt(ReadRegValString(RegValKnightValue, '320'));
  MaterialValue[Bishop] := StrToInt(ReadRegValString(RegValBishopValue, '320'));
  MaterialValue[Rook] := StrToInt(ReadRegValString(RegValRookValue, '520'));
  MaterialValue[Queen] := StrToInt(ReadRegValString(RegValQueenValue, '1000'));
  MaterialValue[King] := 0;
  DoublePawnScore := StrToInt(ReadRegValString(RegValDoublePawnScore, '-15'));
  IsolatedPawnScore := StrToInt(ReadRegValString(RegValIsolatedPawnScore, '-10'));
  BishopPairScore := StrToInt(ReadRegValString(RegValBishopPairScore, '30'));
  KnightStrongScore := StrToInt(ReadRegValString(RegValKnightStrongScore, '10'));
  BishopStrongScore := StrToInt(ReadRegValString(RegValBishopStrongScore, '15'));
  BishopBlocksPawnScore := StrToInt(ReadRegValString(RegValBishopBlocksPawnScore, '-100'));
  BishopLowMobilityScore := StrToInt(ReadRegValString(RegValBishopLowMobilityScore, '-5'));
  RookHalfOpenFileScore := StrToInt(ReadRegValString(RegValRookHalfOpenFileScore, '10'));
  RookOpenFileScore := StrToInt(ReadRegValString(RegValRookOpenFileScore, '20'));
  Rook7thRankScore := StrToInt(ReadRegValString(RegValRook7thRankScore, '25'));
  WeakPinScore := StrToInt(ReadRegValString(RegValWeakPinScore, '5'));
  StrongPinScore := StrToInt(ReadRegValString(RegValStrongPinScore, '10'));
  //
  KnightWeakScore := StrToInt(ReadRegValString(RegValKnightWeakScore, '-60'));
  BishopWeakScore := StrToInt(ReadRegValString(RegValBishopWeakScore, '-70'));
  RookWeakScore := StrToInt(ReadRegValString(RegValRookWeakScore, '-80'));
  QueenWeakScore := StrToInt(ReadRegValString(RegValQueenWeakScore, '-90'));
  KingWeakScore := StrToInt(ReadRegValString(RegValKingWeakScore, '-10'));
  KingShortCastledScore := StrToInt(ReadRegValString(RegValKingShortCastledScore, '50'));
  KingLongCastledScore := StrToInt(ReadRegValString(RegValKingLongCastledScore, '40'));
	KingLostCastlingRightsScore := StrToInt(ReadRegValString(RegValKingLostCastlingRightsScore, '-50'));
  KingOpenFileScore := StrToInt(ReadRegValString(RegValKingOpenFileScore, '-50'));
  //
  PawnExchangeScore := StrToInt(ReadRegValString(RegValPawnExchangeScore, '15'));
  QueenExchangeScore := StrToInt(ReadRegValString(RegValQueenExchangeScore, '-25'));
	//
  Options.TimeLimit := StrToInt(ReadRegValString(RegValTimeLimit, '30'));
  Options.DepthLimit := StrToInt(ReadRegValString(RegValDepthLimit, '32'));
  //
  GameMovesFirstPeriod := StrToInt(ReadRegValString(RegValGameMovesFirstPeriod, '40'));
  GameTimeFirstPeriod := StrToInt(ReadRegValString(RegValGameTimeFirstPeriod, '60'));
  GameMovesSecondPeriod := StrToInt(ReadRegValString(RegValGameMovesSecondPeriod, '20'));
  GameTimeSecondPeriod := StrToInt(ReadRegValString(RegValGameTimeSecondPeriod, '30'));
  //
  WhiteBackGroundColor := StrToInt(ReadRegValString(RegValWhiteBackGroundColor, IntToStr(clLtGray)));
  BlackBackGroundColor := StrToInt(ReadRegValString(RegValBlackBackGroundColor, IntToStr(clDkGray)));
  BoardGrid.Font.Size := FontSize;
	AdjustStringGrid(BoardGrid, AllColumns, True, 'P');
  AdjustStringGrid(MovesGrid, 0, False, ' 1234567 ');
  AdjustStringGrid(GameMovesGrid, 0, False, ' 123 ');
  AdjustStringGrid(GameMovesGrid, 1, False, ' 1234567 ');
  AdjustStringGrid(GameMovesGrid, 2, False, ' 1234567 ');
  SetCacheLength;
  SetQuiesCacheLength;
  SetScoreCacheLength;
  ClearCache;
  ClearQuiesCache;
  ClearScoreCache;
  //
  SearchStatus := SearchStopped;
  SearchThread := TSearchThread.Create(False);
  GameMode := HumanHuman;
	NewItemClick(nil);
end;

procedure TMainForm.ParametersItemClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	ParametersForm.ShowModal;
	ClearCache;
  ClearScoreCache;
end;

procedure TMainForm.ClearMovesItemClick(Sender: TObject);
var
	iRow, iCol: Integer;
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	iRow := GameMovesGrid.Row;
  iCol := GameMovesGrid.Col;
  NextGameCol := iCol;
  NextGameRow := iRow;
  while GameMovesGrid.Cells[iCol, iRow] <> '' do
  begin
  	GameMovesGrid.Cells[iCol, iRow] := '';
    Inc(iCol);
    if iCol > 2 then
    begin
    	Inc(iRow);
      iCol := 1;
    end;
  end;
  ReadGameMoves;
end;

procedure TMainForm.AboutItemClick(Sender: TObject);
begin
	AboutForm.ShowModal;
end;

procedure TMainForm.ShowVisitItemClick(Sender: TObject);
begin
	DebugVisitForm.ShowModal;
end;

procedure TMainForm.ShowScoreItemClick(Sender: TObject);
begin
	DebugScoreForm.Show;
end;

procedure TMainForm.SettingsItemClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	SettingsForm.ShowModal;
	ClearCache;
  ClearScoreCache;
end;

procedure TMainForm.StartSearchButtonClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
	ReadGameMoves;
end;

procedure TMainForm.AbortSearchButtonClick(Sender: TObject);
begin
	AbortSearch;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	ExitItemClick(nil);
end;

procedure TMainForm.TimeUsedEditClick(Sender: TObject);
begin
	TimeUsedForm.ShowModal;
end;

procedure TMainForm.GameMovesGridClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
  GameMode := HumanHuman;
  NextGameCol := GameMovesGrid.Col;
	NextGameRow := GameMovesGrid.Row;
  ReadGameMoves;
end;

procedure TMainForm.BrowseButtonClick(Sender: TObject);
begin
	if ProcessingMessages then
  	Exit;
	AbortSearch;
  GameMode := HumanHuman;
  Inc(NextGameCol);
  if NextGameCol > 2 then
  begin
  	Inc(NextGameRow);
    NextGameCol := 1;
  end;
  ReadGameMoves;
end;

procedure TMainForm.Markregistryold1Click(Sender: TObject);
begin
	WriteRegValString(RegValVersion, '1');
end;

end.

