{
ZZZZZZ 5.0, a chess program.
Copyright (C) 1998 G. Wiesenekker
E-mail: gijsbert.wiesenekker@wxs.nl
http://home.wxs.nl/~gijsbert.wiesenekker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit Score;

{$MODE Delphi}

interface

uses Chess;

const
	InfiniteScore = 1000000000;
  MateScore =     10000000;
  EndgameScore =  100000;
	PlusInfinity: ScoreRecord = (
  	MatScore: InfiniteScore;
    PosScore: 0;
    VisitScore: 0;);
  MinusInfinity: ScoreRecord = (
  	MatScore: -InfiniteScore;
    PosScore: 0;
    VisitScore: 0;);
  WonScore: ScoreRecord = (
  	MatScore: MateScore;
    PosScore: 0;
    VisitScore: 0;);
  WonLowerBound: ScoreRecord = (
  	MatScore: MateScore - NodesMax;
    PosScore: 0;
    VisitScore: 0;);
  DrawScore: ScoreRecord = (
  	MatScore: 0;
    PosScore: 0;
    VisitScore: 0;);
  LostScore: ScoreRecord = (
    MatScore: -MateScore;
    PosScore: 0;
    VisitScore: 0;);
  LostUpperBound: ScoreRecord = (
  	MatScore: -MateScore + NodesMax;
    PosScore: 0;
    VisitScore: 0;);
type
	AttackArray = array [BoardRange] of ShortInt;
  VisitBoard = array [BoardRange] of Integer;
 	NormalVisitBoard = array [BoardRange] of Integer;
	VisitArray = array [0..8, 0..8] of VisitBoard;
  NormalVisitArray = array[0..8, 0..8] of NormalVisitBoard;
var
	DoublePawnScore, IsolatedPawnScore: Integer;
	BishopPairScore: Integer;
  KnightStrongScore: Integer;
  BishopStrongScore, BishopBlocksPawnScore, BishopLowMobilityScore: Integer;
  RookHalfOpenFileScore, RookOpenFileScore: Integer;
  Rook7thRankScore: Integer;
  WeakPinScore, StrongPinScore: Integer;
  KnightWeakScore, BishopWeakScore, RookWeakScore, QueenWeakScore, KingWeakScore: Integer;
  KingShortCastledScore, KingLongCastledScore: Integer;
  KingLostCastlingRightsScore: Integer;
  KingOpenFileScore: Integer;
  PawnExchangeScore, QueenExchangeScore: Integer;
	MaterialValue: array[BoardKindEnum] of Integer;
	Visit: array[BoardColourEnum, BoardKindEnum] of VisitArray;
  NormalVisit: array[BoardColourEnum, BoardKindEnum] of NormalVisitArray;
  AttackBoard: array[BoardColourEnum] of AttackArray;
  Dangerous: Boolean;

function ScoreToString(Score: ScoreRecord): AnsiString;
procedure FlipScore(Scorea: ScoreRecord; var ScoreB: ScoreRecord);
function CompareScore(ScoreA: ScoreRecord; Oper: Operator; ScoreB: ScoreRecord): Boolean;
procedure SetMaterialScore;
procedure CheckMaterialScore;
procedure SetScore(MyColour, YourColour: BoardColourEnum;
	var Score: ScoreRecord);

implementation

uses Cache, SysUtils, Math, DebugScore;

const
 	FreePawnScore: array[BoardColourEnum, BoardRange] of Integer =
  	((0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0,48,64,64,64,64,64,64,48, 0, 0,
  	  0, 0,24,32,32,32,32,32,32,24, 0, 0,
  	  0, 0,12,16,16,16,16,16,16,12, 0, 0,
  	  0, 0, 6, 8, 8, 8, 8, 8, 8, 6, 0, 0,
  	  0, 0, 3, 4, 4, 4, 4, 4, 4, 3, 0, 0,
  	  0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0,
  	  0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
  	 (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0,
  	  0, 0, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0,
  	  0, 0, 3, 4, 4, 4, 4, 4, 4, 3, 0, 0,
  	  0, 0, 6, 8, 8, 8, 8, 8, 8, 6, 0, 0,
  	  0, 0,12,16,16,16,16,16,16,12, 0, 0,
  	  0, 0,24,32,32,32,32,32,32,24, 0, 0,
  	  0, 0,48,64,64,64,64,64,64,48, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  	  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
	LoneKingScore: array[BoardRange] of Integer =
		(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		 0, 0,-32,-16, -8, -8, -8, -8,-16,-32, 0, 0,
		 0, 0,-16, -8, -4, -4, -4, -4, -8,-16, 0, 0,
		 0, 0, -8, -4, -4, -2, -2, -4, -4, -8, 0, 0,
		 0, 0, -8, -4, -2, -1, -1, -2, -4, -8, 0, 0,
		 0, 0, -8, -4, -2, -1, -1, -2, -4, -8, 0, 0,
		 0, 0, -8, -4, -4, -2, -2, -4, -4, -8, 0, 0,
		 0, 0,-16, -8, -4, -4, -4, -4, -8,-16, 0, 0,
		 0, 0,-32,-16, -8, -8, -8, -8,-16,-32, 0, 0,
		 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
var
  OpenFile: array[BoardColourEnum, FileRange] of Boolean;
  nWhitePawn, nBlackPawn: Integer;
  TruePieceCount: array[BoardColourEnum, BoardKindEnum] of ExtListRange;
  TotalPieceCount: array[BoardColourEnum] of ExtListRange;
  PinBoard: array[BoardRange] of Boolean;

procedure MyPrintf(TheString: AnsiString);
begin
	DebugScoreForm.OutputMemo.Lines.Add(TheString);
end;

procedure ClearAttackBoard(var AttackBoard: AttackArray);
var
	i: ExtBoardRange;
begin
  for i := A8 to H1 do
  	AttackBoard[i] := 0;
end;

procedure ReadVisit;
var
  iKind: BoardKindEnum;
	nWhitePawn, nBlackPawn, iBoard: Integer;
  TheFile: TextFile;
  Temp: Integer;
  procedure AverageVisit(var TheVisit: VisitBoard; var TheNormalVisit: NormalVisitBoard);
  var
  	//else overflow!
  	Sum: Int64;
    iBoard: Integer;
  begin
    Sum := 0;
    for iBoard := 1 to 64 do
    	Sum := Max(Sum, TheVisit[LinearToBoardIndex[iBoard]]);
		if Sum > 0 then
    begin
    	for iBoard := 1 to 64 do
    		TheVisit[LinearToBoardIndex[iBoard]] :=
        	TheVisit[LinearToBoardIndex[iBoard]] * Int64(100) div Sum;
    end;
    //subtract average
    Sum := 0;
    for iBoard := 1 to 64 do
    	Inc(Sum, TheVisit[LinearToBoardIndex[iBoard]]);
    Sum := Sum div 64;
    for iBoard := 1 to 64 do
    begin
    	Dec(TheVisit[LinearToBoardIndex[iBoard]], Sum);
      if (TheVisit[LinearToBoardIndex[iBoard]] > Sum) then
        TheNormalVisit[LinearToBoardIndex[iBoard]] := (TheVisit[LinearToBoardIndex[iBoard]] - Sum) div 10
      else
        TheNormalVisit[LinearToBoardIndex[iBoard]] := 0;
    end;
  end;
begin
	AssignFile(TheFile, 'visit.txt');
	Reset(TheFile);
	for nWhitePawn := 0 to 8 do
  begin
  	for nBlackPawn := 0 to 8 do
    begin
    	for iKind := Pawn to King do
      begin
        for iBoard := 1 to 64 do
        begin
          read(TheFile, Temp);
          Visit[White, iKind][nWhitePawn, nBlackPawn][LinearToBoardIndex[iBoard]] := Temp;
          Visit[Black, iKind][nWhitePawn, nBlackPawn][InvLinearToBoardIndex[iBoard]] := Temp;
        end;
      	readln(TheFile);
        AverageVisit(Visit[White, iKind][nWhitePawn, nBlackPawn],
        	NormalVisit[White, iKind][nWhitePawn, nBlackPawn]);
        AverageVisit(Visit[Black, iKind][nWhitePawn, nBlackPawn],
        	NormalVisit[Black, iKind][nWhitePawn, nBlackPawn]);
			end;
    end;
  end;
end;

function ScoreToString(Score: ScoreRecord): AnsiString;
begin
	Result := IntToStr(Score.MatScore) + '/' + IntToStr(Score.PosScore) +
  	'/' + IntToStr(Score.VisitScore);
end;

procedure FlipScore(Scorea: ScoreRecord; var ScoreB: ScoreRecord);
begin
	ScoreB.MatScore := -ScoreA.MatScore;
	ScoreB.PosScore := -ScoreA.PosScore;
	ScoreB.VisitScore := -ScoreA.VisitScore;
end;

function CompareScore(ScoreA: ScoreRecord; Oper: Operator; ScoreB: ScoreRecord): Boolean;
var
  Temp: ScoreRecord;
  MatScoreA, PosScoreA, VisitScoreA: Integer;
  MatScoreB, PosScoreB, VisitScoreB: Integer;
begin
	if Oper = LT then
  begin
  	Oper := GT;
    Temp := ScoreA;
    ScoreA := ScoreB;
    ScoreB := Temp;
  end
  else if Oper = LE then
  begin
  	Oper := GE;
    Temp := ScoreA;
    ScoreA := ScoreB;
    ScoreB := Temp;
  end;

  Result := False;

  MatScoreA := ScoreA.MatScore;
  PosScoreA := ScoreA.PosScore;
  VisitScoreA := ScoreA.VisitScore;
  MatScoreB := ScoreB.MatScore;
  PosScoreB := ScoreB.PosScore;
  VisitScoreB := ScoreB.VisitScore;
  
  if Oper = GT then
  begin
  	if MatScoreA > MatScoreB then
    begin
    	Result := True;
    end
    else if MatScoreA = MatScoreB then
    begin
    	if PosScoreA > PosScoreB then
      begin
      	Result := True;
      end
      else if PosScoreA = PosScoreB then
      begin
      	if VisitScoreA > VisitScoreB then
        begin
        	Result := True;
        end
      end;
    end;
  end
  else if Oper = GE then
  begin
  	if MatScoreA > MatScoreB then
    begin
    	Result := True;
    end
    else if MatScoreA = MatScoreB then
    begin
    	if PosScoreA > PosScoreB then
      begin
      	Result := True;
      end
      else if PosScoreA = PosScoreB then
      begin
      	if VisitScoreA >= VisitScoreB then
        begin
        	Result := True
        end
      end;
    end;
  end
  else if Oper = EQ then
  begin
  	if (MatScoreA = MatScoreB) and (PosScoreA = PosScoreB) and
       (VisitScoreA = VisitScoreB) then
    	Result := True;
  end;
end;

procedure SetMaterialScore;
var
	Colour: BoardColourEnum;
  Kind: BoardKindEnum;
  i: ExtBoardRange;
begin
  for Colour := White to Black do
  begin
  	MaterialScore[Colour] := 0;
  	for Kind := Pawn to Queen do
    begin
    	for i := 1 to PieceCount[Colour, Kind] do
      begin
      	with PieceList[Colour, Kind, i] do
        begin
        	if ListWhere = Low(ExtBoardRange) then
        		continue;
        	Inc(MaterialScore[Colour], MaterialValue[Kind]);
      	end;
    	end;
    end;
  end;
end;

procedure CheckMaterialScore;
var
	Colour: BoardColourEnum;
  Kind: BoardKindEnum;
  i: ExtBoardRange;
  Temp: Integer;
begin
  for Colour := White to Black do
  begin
		Temp := 0;
  	for Kind := Pawn to Queen do
    begin
    	for i := 1 to PieceCount[Colour, Kind] do
      begin
      	with PieceList[Colour, Kind, i] do
        begin
        	if ListWhere = Low(ExtBoardRange) then
        		continue;
        	Inc(Temp, MaterialValue[Kind]);
      	end;
    	end;
    end;
  	if (Temp <> MaterialScore[Colour]) then
  		raise EChessError.Create('Material error.' +
      	' Material: ' + IntToStr(MaterialScore[Colour]) +
    		' Material from lists: ' + IntToStr(Temp));
  end;
end;

procedure SetAttackBoard(MyColour, YourColour: BoardColourEnum;
	var AttackBoard: AttackArray; var TempPos: Integer; var TempVisit: Integer);
var
	iList: ExtListRange;
	iBoard: ExtBoardRange;

  function Xray(Dir: Integer; jBoard: ExtBoardRange): Boolean;
  var
  	iDir: Integer;
  begin
  	Result := True;
  	if Board[jBoard].ListKind = Bishop then
    begin
    	for iDir := 1 to 4 do
      	if BishopDir[iDir] = Dir then
        	Exit;
    end
    else if Board[jBoard].ListKind = Rook then
    begin
    	for iDir := 1 to 4 do
      	if RookDir[iDir] = Dir then
        	Exit;
    end
    else if Board[jBoard].ListKind = Queen then
    begin
    	Exit;
    end;
    Result := False;
  end;

	procedure SetPawnAttack(Dir1, Dir2: Integer);
  var
  	jBoard: ExtBoardRange;
  begin
  	jBoard := iBoard + Dir1;
    Inc(AttackBoard[jBoard]);
    jBoard := iBoard + Dir2;
    Inc(AttackBoard[jBoard]);
  end;

	procedure SetKnightKingAttack(Dir: DirArray; NDir: DirRange);
  var
  	jBoard: ExtBoardRange;
  	iDir: Integer;
  begin
    for iDir := 1 to NDir do
    begin
    	jBoard := iBoard + Dir[iDir];
			Inc(AttackBoard[jBoard]);
    end;
  end;

	procedure SetBishopRookQueenAttack(Dir: DirArray; NDir: DirRange);
  var
  	jBoard, kBoard: BoardRange;
  	iDir: Integer;
  begin
    for iDir := 1 to NDir do
    begin
    	jBoard := iBoard + Dir[iDir];
    	while Board[jBoard].ListKind = Empty do
      begin
      	Inc(AttackBoard[jBoard]);
        Inc(jBoard, Dir[iDir]);
      end;
			Inc(AttackBoard[jBoard]);
      //xray
      if Board[jBoard].ListColour = MyColour then
      begin
      	if Xray(Dir[iDir], jBoard) then
        begin
       		Inc(jBoard, Dir[iDir]);
          while Board[jBoard].ListKind = Empty do
          begin
            Inc(AttackBoard[jBoard]);
            Inc(jBoard, Dir[iDir]);
          end;
          Inc(AttackBoard[jBoard]);
        end;
      end
      else if (Board[iBoard].ListKind <> Queen) and (Board[jBoard].ListColour = YourColour) and
      	(Board[jBoard].ListKind <> Pawn) then
      begin
        kBoard := jBoard + Dir[iDir];
        while Board[kBoard].ListKind = Empty do
          Inc(kBoard, Dir[iDir]);
				if (Board[kBoard].ListColour = YourColour) and (Board[kBoard].ListKind = Queen) or
           (Board[kBoard].ListKind = King) then
        begin
        	if (Board[jBoard].ListKind <> Queen) and (Board[iBoard].ListKind <> Board[jBoard].ListKind) then
          begin
            if ShowScore then
            begin
              MyPrintf(ColourToString(MyColour) + ' strongly pins piece on ' + BoardIndexToString(jBoard));
            end;
          	Inc(TempPos, StrongPinScore);
            PinBoard[jBoard] := True;
          end
          else
          begin
            if ShowScore then
            begin
              MyPrintf(ColourToString(MyColour) + ' weakly pins piece on ' + BoardIndexToString(jBoard));
            end;
          	Inc(TempPos, WeakPinScore);
          end;
        end;
      end;
    end;
  end;

begin
	ClearAttackBoard(AttackBoard);
  TruePieceCount[MyColour, Pawn] := 0;
	for iList := 1 to PieceCount[MyColour, Pawn] do
  begin
  	iBoard := PieceList[MyColour, Pawn, iList].ListWhere;
    if iBoard = Low(ExtBoardRange) then continue;
  	Inc(TruePieceCount[MyColour, Pawn]);
    Inc(TempVisit, Visit[MyColour, Pawn][nWhitePawn, nBlackPawn][iBoard]);
    SetPawnAttack(PawnCaptureDir1[MyColour], PawnCaptureDir2[MyColour]);
  end;
  TruePieceCount[MyColour, Knight] := 0;
	for iList := 1 to PieceCount[MyColour, Knight] do
  begin
  	iBoard := PieceList[MyColour, Knight, iList].ListWhere;
    if iBoard = Low(ExtBoardRange) then continue;
  	Inc(TruePieceCount[MyColour, Knight]);
    Inc(TempVisit, Visit[MyColour, Knight][nWhitePawn, nBlackPawn][iBoard]);
    SetKnightKingAttack(KnightDir, 8);
  end;
  TruePieceCount[MyColour, Bishop] := 0;
	for iList := 1 to PieceCount[MyColour, Bishop] do
  begin
  	iBoard := PieceList[MyColour, Bishop, iList].ListWhere;
    if iBoard = Low(ExtBoardRange) then continue;
  	Inc(TruePieceCount[MyColour, Bishop]);
    Inc(TempVisit, Visit[MyColour, Bishop][nWhitePawn, nBlackPawn][iBoard]);
    SetBishopRookQueenAttack(BishopDir, 4);
  end;
  TruePieceCount[MyColour, Rook] := 0;
	for iList := 1 to PieceCount[MyColour, Rook] do
  begin
  	iBoard := PieceList[MyColour, Rook, iList].ListWhere;
    if iBoard = Low(ExtBoardRange) then continue;
  	Inc(TruePieceCount[MyColour, Rook]);
    Inc(TempVisit, Visit[MyColour, Rook][nWhitePawn, nBlackPawn][iBoard]);
    SetBishopRookQueenAttack(RookDir, 4);
  end;
  TruePieceCount[MyColour, Queen] := 0;
	for iList := 1 to PieceCount[MyColour, Queen] do
  begin
  	iBoard := PieceList[MyColour, Queen, iList].ListWhere;
    if iBoard = Low(ExtBoardRange) then continue;
  	Inc(TruePieceCount[MyColour, Queen]);
    SetBishopRookQueenAttack(QueenDir, 8);
  end;
  TruePieceCount[MyColour, King] := 0;
	for iList := 1 to PieceCount[MyColour, King] do
  begin
  	iBoard := PieceList[MyColour, King, iList].ListWhere;
    if iBoard = Low(ExtBoardRange) then continue;
  	Inc(TruePieceCount[MyColour, King]);
    Inc(TempVisit, Visit[MyColour, King][nWhitePawn, nBlackPawn][iBoard]);
    SetKnightKingAttack(KingDir, 8);
	end;
  TotalPieceCount[MyColour] :=
  	TruePieceCount[MyColour, Pawn] +
    TruePieceCount[MyColour, Knight] +
    TruePieceCount[MyColour, Bishop] +
    TruePieceCount[MyColour, Rook] +
    TruePieceCount[MyColour, Queen] +
    TruePieceCount[MyColour, King];
end;

function FreePawn(MyColour, YourColour: BoardColourEnum;
	iBoard: ExtBoardRange): Boolean;
var
	jBoard: ExtBoardRange;
begin
	Result := False;
  while True do
  begin
  	jBoard := iBoard + PawnCaptureDir1[MyColour];
    if (Board[jBoard].ListKind = Pawn) and (Board[jBoard].ListColour = YourColour) then
    	Exit;
  	jBoard := iBoard + PawnCaptureDir2[MyColour];
    if (Board[jBoard].ListKind = Pawn) and (Board[jBoard].ListColour = YourColour) then
    	Exit;
    Inc(iBoard, PawnMoveDir[MyColour]);
    if Board[iBoard].ListKind = Pawn then
    	Exit;
    if Board[iBoard].ListKind = Side then
    	Break;
    //any other kind: scan
  end;
  Result := True;
end;

procedure PawnPosScore(MyColour, YourColour: BoardColourEnum;
	var TempPos: Integer);
var
	iList: ExtListRange;
  iFile: ExtFileRange;
  PawnCount: array[FileRange] of Integer;
	IsolatedPawn: Boolean;
begin
  for iFile := 1 to 8 do
  	PawnCount[iFile] := 0;
  for iList := 1 to PieceCount[MyColour, Pawn] do
  begin
    with PieceList[MyColour, Pawn, iList] do
    begin
      if ListWhere = Low(ExtBoardRange) then
        continue;
      if NormalVisit[MyColour, Pawn][nWhitePawn, nBlackPawn][ListWhere] > 0 then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' pawn on ' + BoardIndexToString(ListWhere) +
          	' is on normal square');
        Inc(TempPos, NormalVisit[MyColour, Pawn][nWhitePawn, nBlackPawn][ListWhere]);
      end;
      OpenFile[MyColour, FileNumber[ListWhere]] := False;
      Inc(PawnCount[FileNumber[ListWhere]]);
      if FreePawn(MyColour, YourColour, ListWhere) then
      begin
      	if ShowScore then
        begin
        	MyPrintf(ColourToString(MyColour) + ' free pawn on ' + BoardIndexToString(ListWhere));
        end;
        Inc(TempPos, FreePawnScore[MyColour, ListWhere]);
      end;
    end;
  end;
  for iFile := 1 to 8 do
  begin
  	if PawnCount[iFile] > 1 then
    begin
    	if ShowScore then
      begin
    		MyPrintf(ColourToString(MyColour) + ' double pawn on file ' + IntToStr(iFile));
      end;
    	Inc(TempPos, DoublePawnScore);
    end;
    if PawnCount[iFile] >= 1 then
    begin
      IsolatedPawn := True;
      if iFile = 1 then
      begin
      	if PawnCount[2] > 0 then
        	IsolatedPawn := False;
      end
      else if iFile = 8 then
      begin
      	if PawnCount[7] > 0 then
        	IsolatedPawn := False;
      end
      else
      begin
      	if (PawnCount[iFile - 1] > 0) or (PawnCount[iFile + 1] > 0) then
        	IsolatedPawn := False;
      end;
      if IsolatedPawn then
      begin
        if ShowScore then
        begin
          MyPrintf(ColourToString(MyColour) + ' isolated pawn on file ' + IntToStr(iFile));
        end;
        Inc(TempPos, IsolatedPawnScore);
      end
    end
  end;
end;

function FreePiece(MyColour, YourColour: BoardColourEnum;
	iBoard: ExtBoardRange): Boolean;
var
	jBoard: ExtBoardRange;
begin
	Result := False;
  while True do
  begin
  	jBoard := iBoard + PawnCaptureDir1[MyColour];
    if (Board[jBoard].ListKind = Pawn) and (Board[jBoard].ListColour = YourColour) then
    	Exit;
  	jBoard := iBoard + PawnCaptureDir2[MyColour];
    if (Board[jBoard].ListKind = Pawn) and (Board[jBoard].ListColour = YourColour) then
    	Exit;
    Inc(iBoard, PawnMoveDir[MyColour]);
    if Board[iBoard].ListKind = Side then
    	Break;
  end;
  Result := True;
end;

procedure KnightPosScore(MyColour, YourColour: BoardColourEnum;
	var TempPos: Integer);
var
	iList: ExtListRange;
  jBoard: ExtBoardRange;
  KnightWeak: Boolean;
  nCount: Integer;

	procedure KnightMobility(iBoard: ExtBoardRange);
  var
  	jBoard: ExtBoardRange;
  	iDir: Integer;
  begin
    KnightWeak := True;
    nCount := 0;
    for iDir := 1 to 8 do
    begin
    	jBoard := iBoard + KnightDir[iDir];
      if Board[jBoard].ListKind = Side then
      	Continue;
      if Board[jBoard].ListColour = MyColour then
				KnightWeak := False
      //else YourColour or Empty
      else if AttackBoard[YourColour, jBoard] < AttackBoard[MyColour, jBoard] then
      begin
      	KnightWeak := False;
        Inc(nCount);
      end;
      //else AttackBoard[YourColour, jBoard] >= AttackBoard[MyColour, jBoard]
      //Since AttackBoard[MyColour, jBoard] includes Knight itself,
      //there is a problem, so we leave KnightWeak at True.
    end;
  end;

begin
  for iList := 1 to PieceCount[MyColour, Knight] do
  begin
    with PieceList[MyColour, Knight, iList] do
    begin
      if ListWhere = Low(ExtBoardRange) then
        continue;
      KnightMobility(ListWhere);
      if KnightWeak then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' weak knight on ' + BoardIndexToString(ListWhere));
        Inc(TempPos, KnightWeakScore);
      end
      else if NormalVisit[MyColour, Knight][nWhitePawn, nBlackPawn][ListWhere] > 0 then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' knight on ' + BoardIndexToString(ListWhere) +
          	' is on normal square');
        Inc(TempPos);
      end;
      if FreePiece(MyColour, YourColour, ListWhere) then
      begin
      	jBoard := ListWhere - PawnCaptureDir1[MyColour];
        if (Board[jBoard].ListColour = MyColour) and (Board[jBoard].ListKind = Pawn) then
        begin
      		if ShowScore then
        	begin
        		MyPrintf(ColourToString(MyColour) + ' strong knight on ' + BoardIndexToString(ListWhere));
          end;
          Inc(TempPos, KnightStrongScore);
        end;
      	jBoard := ListWhere - PawnCaptureDir2[MyColour];
        if (Board[jBoard].ListColour = MyColour) and (Board[jBoard].ListKind = Pawn) then
        begin
      		if ShowScore then
        	begin
        		MyPrintf(ColourToString(MyColour) + ' strong knight on ' + BoardIndexToString(ListWhere));
          end;
          Inc(TempPos, KnightStrongScore);
        end;
      end
      else if PinBoard[ListWhere] and (AttackBoard[MyColour, ListWhere] = 0) then
      begin
      	Dangerous := True;
      end;
    end;
  end;
end;

procedure BishopPosScore(MyColour, YourColour: BoardColourEnum;
	var TempPos: Integer);
var
	iList: ExtListRange;
  jBoard: ExtBoardRange;
  nBishop: Integer;
  BishopWeak: Boolean;
  nCount: Integer;

	procedure BishopMobility(iBoard: ExtBoardRange);
  var
  	jBoard: ExtBoardRange;
  	iDir: Integer;
  begin
		BishopWeak := True;
    nCount := 0;
    for iDir := 1 to 4 do
    begin
    	jBoard := iBoard + BishopDir[iDir];
    	while Board[jBoard].ListKind = Empty do
      begin
				BishopWeak := False;
	      if AttackBoard[YourColour, jBoard] < AttackBoard[MyColour, jBoard] then
        	Inc(nCount);
        Inc(jBoard, BishopDir[iDir]);
      end;
      if Board[jBoard].ListKind = Side then
      	Continue;
      if Board[jBoard].ListColour = MyColour then
        BishopWeak := False
      else if AttackBoard[YourColour, jBoard] < AttackBoard[MyColour, jBoard] then
      begin
      	BishopWeak := False;
        Inc(nCount);
      end;
    end;
  end;

  function BishopBlocksCenterPawn(iBoard: ExtBoardRange): Boolean;
  var
  	jBoard: ExtBoardRange;
  begin
  	Result := True;
  	jBoard := iBoard - PawnMoveDir[MyColour];
    //quick hack
    if MyColour = White then
    begin
    	if ((jBoard = D2) or (jBoard = E2)) and not Board[jBoard].ListMoved then
				Exit;
    end
    else if MyColour = Black then
    begin
    	if ((jBoard = D7) or (jBoard = E7)) and not Board[jBoard].ListMoved then
				Exit;
    end;
    Result := False;
  end;

begin
  nBishop := 0;
  for iList := 1 to PieceCount[MyColour, Bishop] do
  begin
    with PieceList[MyColour, Bishop, iList] do
    begin
      if ListWhere = Low(ExtBoardRange) then
        continue;
      Inc(nBishop);
      BishopMobility(ListWhere);
      if BishopWeak then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' weak bishop on ' + BoardIndexToString(ListWhere));
        Inc(TempPos, BishopWeakScore);
      end
      else if NormalVisit[MyColour, Bishop][nWhitePawn, nBlackPawn][ListWhere] > 0 then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' bishop on ' + BoardIndexToString(ListWhere) +
          	' is on normal square');
        Inc(TempPos);
      end
      else if nCount < 3 then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' bishop on ' + BoardIndexToString(ListWhere) +
          	' has low mobility');
        Inc(TempPos, BishopLowMobilityScore);
      end;
      if BishopBlocksCenterPawn(ListWhere) then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' bishop blocks pawn on ' + BoardIndexToString(ListWhere));
        Inc(TempPos, BishopBlocksPawnScore);
      end;
      if FreePiece(MyColour, YourColour, ListWhere) then
      begin
      	jBoard := ListWhere - PawnCaptureDir1[MyColour];
        if (Board[jBoard].ListColour = MyColour) and (Board[jBoard].ListKind = Pawn) then
        begin
      		if ShowScore then
        	begin
        		MyPrintf(ColourToString(MyColour) + ' strong bishop on ' + BoardIndexToString(ListWhere));
          end;
          Inc(TempPos, BishopStrongScore);
        end;
      	jBoard := ListWhere - PawnCaptureDir2[MyColour];
        if (Board[jBoard].ListColour = MyColour) and (Board[jBoard].ListKind = Pawn) then
        begin
      		if ShowScore then
        	begin
        		MyPrintf(ColourToString(MyColour) + ' strong bishop on ' +
          		BoardIndexToString(ListWhere));
          end;
          Inc(TempPos, BishopStrongScore);
        end;
      end
      else if PinBoard[ListWhere] and (AttackBoard[MyColour, ListWhere] = 0) then
      begin
      	Dangerous := True;
      end;
    end;
  end;
  if nBishop > 1 then
  begin
  	if ShowScore then
    begin
    	MyPrintf(ColourToString(MyColour) + ' has at least two bishops');
    end;
    Inc(TempPos, BishopPairScore);
  end;
end;

procedure RookPosScore(MyColour, YourColour: BoardColourEnum;
	var TempPos: Integer);
var
	iList: ExtListRange;
begin
  for iList := 1 to PieceCount[MyColour, Rook] do
  begin
    with PieceList[MyColour, Rook, iList] do
    begin
      if ListWhere = Low(ExtBoardRange) then
        continue;
      if RankNumber[MyColour, ListWhere] = 7 then
      begin
        if ShowScore then
        begin
          MyPrintf(ColourToString(MyColour) + ' rook on 7th rank ' +
            BoardIndexToString(ListWhere));
        end;
        Inc(TempPos, Rook7thRankScore);
      end;
      if OpenFile[MyColour, FileNumber[ListWHere]] then
      begin
      	if OpenFile[YourColour, FileNumber[ListWhere]] then
        begin
          if ShowScore then
          begin
            MyPrintf(ColourToString(MyColour) + ' rook on open file ' +
              BoardIndexToString(ListWhere));
          end;
          Inc(TempPos, RookOpenFileScore);
        end
        else
        begin
          if ShowScore then
          begin
            MyPrintf(ColourToString(MyColour) + ' rook on half open file ' +
              BoardIndexToString(ListWhere));
          end;
          Inc(TempPos, RookHalfOpenFileScore);
        end;
      end;
    end;
  end;
end;

procedure QueenPosScore(MyColour, YourColour: BoardColourEnum;
	var TempPos: Integer);
var
	iList: ExtListRange;
begin
  for iList := 1 to PieceCount[MyColour, Queen] do
  begin
    with PieceList[MyColour, Queen, iList] do
    begin
      if ListWhere = Low(ExtBoardRange) then
        continue;
			if NormalVisit[MyColour, Queen][nWhitePawn, nBlackPawn][ListWhere] > 0 then
      begin
      	if ShowScore then
        	MyPrintf(ColourToString(MyColour) + ' queen on ' + BoardIndexToString(ListWhere) +
          	' is on normal square');
    		Inc(TempPos, NormalVisit[MyColour, Queen][nWhitePawn, nBlackPawn][ListWhere]);
      end;
    end;
  end;
end;

procedure KingPosScore(MyColour, YourColour: BoardColourEnum;
	var TempPos: Integer);
var
  jBoard: ExtBoardRange;
  iDir: Integer;
  function ShortCastlePawns(P1, P2, P3, P4: ExtBoardRange): Boolean;
  var
  	nPawn: Integer;
  begin
    nPawn := 0;
    if (Board[P1].ListColour = MyColour) and (Board[P1].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P2].ListColour = MyColour) and (Board[P2].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P3].ListColour = MyColour) and (Board[P3].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P4].ListColour = MyColour) and (Board[P4].ListKind = Pawn) then
    	Inc(nPawn);
    if nPawn > 1 then
    	Result := True
    else
    	Result := False;
  end;
  function HasShortCastled(X1, X2, X3, X4: ExtBoardRange): Boolean;
  var
		nKing, nRook: Integer;
  begin
    nKing := 0;
    if (Board[X1].ListColour = MyColour) and (Board[X1].ListKind = King) then
    	Inc(nKing);
    if (Board[X2].ListColour = MyColour) and (Board[X2].ListKind = King) then
    	Inc(nKing);
    if (Board[X3].ListColour = MyColour) and (Board[X3].ListKind = King) then
    	Inc(nKing);
    if (Board[X4].ListColour = MyColour) and (Board[X4].ListKind = King) then
    	Inc(nKing);
    nRook := 0;
    if (Board[X1].ListColour = MyColour) and (Board[X1].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X2].ListColour = MyColour) and (Board[X2].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X3].ListColour = MyColour) and (Board[X3].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X4].ListColour = MyColour) and (Board[X4].ListKind = Rook) then
    	Inc(nRook);
    if (nKing = 1) and (nRook = 0) then
    	Result := True
    else
    	Result := False;
  end;
  function LongCastlePawns(P1, P2, P3, P4, P5: ExtBoardRange): Boolean;
  var
  	nPawn: Integer;
  begin
    nPawn := 0;
    if (Board[P1].ListColour = MyColour) and (Board[P1].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P2].ListColour = MyColour) and (Board[P2].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P3].ListColour = MyColour) and (Board[P3].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P4].ListColour = MyColour) and (Board[P4].ListKind = Pawn) then
    	Inc(nPawn);
    if (Board[P5].ListColour = MyColour) and (Board[P5].ListKind = Pawn) then
    	Inc(nPawn);
    if nPawn > 2 then
    	Result := True
    else
    	Result := False;
  end;
  function HasLongCastled(X1, X2, X3, X4, X5: ExtBoardRange): Boolean;
  var
		nKing, nRook: Integer;
  begin
    nKing := 0;
    if (Board[X1].ListColour = MyColour) and (Board[X1].ListKind = King) then
    	Inc(nKing);
    if (Board[X2].ListColour = MyColour) and (Board[X2].ListKind = King) then
    	Inc(nKing);
    if (Board[X3].ListColour = MyColour) and (Board[X3].ListKind = King) then
    	Inc(nKing);
    if (Board[X4].ListColour = MyColour) and (Board[X4].ListKind = King) then
    	Inc(nKing);
    if (Board[X5].ListColour = MyColour) and (Board[X5].ListKind = King) then
    	Inc(nKing);
    nRook := 0;
    if (Board[X1].ListColour = MyColour) and (Board[X1].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X2].ListColour = MyColour) and (Board[X2].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X3].ListColour = MyColour) and (Board[X3].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X4].ListColour = MyColour) and (Board[X4].ListKind = Rook) then
    	Inc(nRook);
    if (Board[X5].ListColour = MyColour) and (Board[X5].ListKind = Rook) then
    	Inc(nRook);
    if (nKing = 1) and (nRook = 0) then
    	Result := True
    else
    	Result := False;
  end;
begin
  with PieceList[MyColour, King, 1] do
  begin
    if NormalVisit[MyColour, King][nWhitePawn, nBlackPawn][ListWhere] > 0 then
    begin
      if ShowScore then
        MyPrintf(ColourToString(MyColour) + ' king on ' + BoardIndexToString(ListWhere) +
          ' is on normal square');
      Inc(TempPos);
    end;
    if OpenFile[MyColour, FileNumber[ListWhere]] and OpenFile[YourColour, FileNumber[ListWhere]] then
    begin
      if ShowScore then
        MyPrintf(ColourToString(MyColour) + ' king on open file ' +
          BoardIndexToString(ListWhere));
      Inc(TempPos, KingOpenFileScore);
    end;
    //castling
    if (TruePieceCount[YourColour, Queen] > 0) and
       (TruePieceCount[YourColour, Rook] > 0) then
    begin
    	if MyColour = White then
      begin
      	if ListMoved then
        begin
        	if ShortCastlePawns(G2, H2, G3, H3) and HasShortCastled(G1, H1, G2, H2) then
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has short castled');
            Inc(TempPos, KingShortCastledScore);
          end
          else if LongCastlePawns(A2, B2, C2, A3, B3) and HasLongCastled(A1, B1, C1, A2, B2) then
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has long castled');
            Inc(TempPos, KingLongCastledScore);
          end
          else
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has lost castling rights');
          	Inc(TempPos, KingLostCastlingRightsScore);
          end;
        end
        else
        begin
        	if (not ShortCastlePawns(G2, H2, G3, H3)) and
             (not LongCastlePawns(A2, B2, C2, A3, B3)) then
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has lost castling rights');
          	Inc(TempPos, KingLostCastlingRightsScore);
          end;
        end;
      end
      else
      begin
      	if ListMoved then
        begin
        	if ShortCastlePawns(G7, H7, G6, H6) and HasShortCastled(G8, H8, G7, H7) then
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has short castled');
            Inc(TempPos, KingShortCastledScore);
          end
          else if LongCastlePawns(A7, B7, C7, A6, B6) and HasLongCastled(A8, B8, C8, A7, B7) then
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has long castled');
            Inc(TempPos, KingLongCastledScore);
          end
          else
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has lost castling rights');
          	Inc(TempPos, KingLostCastlingRightsScore);
          end;
        end
        else
        begin
        	if (not ShortCastlePawns(G7, H7, G6, H6)) and
             (not LongCastlePawns(A7, B7, C7, A6, B6)) then
          begin
            if ShowScore then
              MyPrintf(ColourToString(MyColour) + ' king has lost castling rights');
          	Inc(TempPos, KingLostCastlingRightsScore);
          end;
        end;
      end;
    end;
    for iDir := 1 to 8 do
    begin
    	jBoard := ListWhere + KingDir[iDir];
      if Board[jBoard].ListKind = Side then
      	Continue;
      if AttackBoard[YourColour, jBoard] > AttackBoard[MyColour, jBoard] then
      begin
        if ShowScore then
        begin
          MyPrintf(ColourToString(MyColour) + ' king weak on ' +
            BoardIndexToString(jBoard));
        end;
      	Inc(TempPos, KingWeakScore);
      end;
    end;
  end;
end;

function CheckDrawEndgame(MyColour, YourColour: BoardColourEnum): Boolean;
begin
	Result := True;
	if TotalPieceCount[YourColour] = 1 then
  begin
  	if TotalPieceCount[MyColour] = 1 then
    begin
    	if ShowScore then
      	MyPrintf('Theoretical draw: King vs. King');
    	Exit;
    end
    else if (TotalPieceCount[MyColour] = 2) then
    begin
    	if (TruePieceCount[MyColour, Knight] = 1) then
    	begin
    		if ShowScore then
      		MyPrintf('Theoretical draw: King and Knight vs. King');
    		Exit;
    	end
    	else if (TruePieceCount[MyColour, Bishop] = 1) then
    	begin
    		if ShowScore then
      		MyPrintf('Theoretical draw: King and Bishop vs. King');
    		Exit;
      end;
    end;
  end;
  Result := False;
end;

function CheckWonEndgame(MyColour, YourCOlour: BoardColourEnum;
	var Temp: Integer): Boolean;
var
	iBoard, jBoard: Integer;
	FileDiff, RankDiff: Integer;
begin
	Result := True;
  if TotalPieceCount[YourColour] = 1 then
  begin
  	if (TotalPieceCount[MyColour] = 2) and
        ((TruePieceCount[MyColour, Rook] = 1) or (TruePieceCount[MyColour, Queen] = 1)) then
  	begin
    	if ShowScore then
      	MyPrintf('Theorical win: King and Rook/Queen vs. King');
      iBoard := PieceList[MyColour, King, 1].ListWhere;
      jBoard := PieceList[YourColour, King, 1].ListWhere;
      FileDiff := FileNumber[iBoard] - FileNumber[jBoard];
      if FileDiff < 0 then
      	FileDiff := -FileDiff;
      RankDiff := RankNumber[MyColour, iBoard] - RankNumber[MyColour, jBoard];
      if RankDiff < 0 then
      	RankDiff := -Rankdiff;
      //sign headache..
      Temp := -LoneKingScore[jBoard] - FileDiff - RankDiff;
      Exit;
    end;
  end;
  Result := False;
end;

procedure SetScore(MyColour, YourColour: BoardColourEnum;
	var Score: ScoreRecord);
var
  TempPos: array[BoardColourEnum] of Integer;
  TempVisit: array[BoardColourEnum] of Integer;
	iCache: Integer;
  iList: ExtListRange;
  iFile: ExtFileRange;
  Temp: Integer;
  ComputerColour, HumanColour: BoardColourEnum;
  i: Integer;
label
	LabelStore;
begin
  Dangerous := False;
  iCache := ReturnScoreCacheIndex;
  with ScoreCache[MyColour][iCache] do
  begin
  	if not ShowScore and Options.UseCache and (ScoreCacheKey = NodeList[Node].NodeFullHashKey) then
    begin
    	Score := ScoreCacheScore;
      Exit;
    end;
  end;
  //
  for i := A8 to H1 do
  	PinBoard[i] := False;
  Score.MatScore := MaterialScore[MyColour] - MaterialScore[YourColour];
  TempPos[White] := 0;
  TempPos[Black] := 0;
  TempVisit[White] := 0;
  TempVisit[Black] := 0;
  //
  //count pawns for visit tables
  nWhitePawn := 0;
  for iList := 1 to PieceCount[White, Pawn] do
  begin
		if PieceList[White, Pawn, iList].ListWhere = Low(ExtBoardRange) then
    	Continue;
    Inc(nWhitePawn);
  end;
  nBlackPawn := 0;
  for iList := 1 to PieceCount[Black, Pawn] do
  begin
		if PieceList[Black, Pawn, iList].ListWhere = Low(ExtBoardRange) then
    	Continue;
    Inc(nBlackPawn);
  end;
	//
  SetAttackBoard(White, Black, AttackBoard[White], TempPos[White], TempVisit[White]);
  SetAttackBoard(Black, White, AttackBoard[Black], TempPos[Black], TempVisit[Black]);
  //
  if ShowScore then
  begin
  	MyPrintf('Piececount white: ' + IntToStr(TotalPieceCount[White]) +
    	'/' + IntToStr(TruePieceCount[White, Pawn]) +
    	'/' + IntToStr(TruePieceCount[White, Knight]) +
    	'/' + IntToStr(TruePieceCount[White, Bishop]) +
    	'/' + IntToStr(TruePieceCount[White, Rook]) +
    	'/' + IntToStr(TruePieceCount[White, Queen]) +
    	'/' + IntToStr(TruePieceCount[White, King]));
  	MyPrintf('Piececount black: ' + IntToStr(TotalPieceCount[Black]) +
    	'/' + IntToStr(TruePieceCount[Black, Pawn]) +
    	'/' + IntToStr(TruePieceCount[Black, Knight]) +
    	'/' + IntToStr(TruePieceCount[Black, Bishop]) +
    	'/' + IntToStr(TruePieceCount[Black, Rook]) +
    	'/' + IntToStr(TruePieceCount[Black, Queen]) +
    	'/' + IntToStr(TruePieceCount[Black, King]));
  end;
  //
  //simple endgames
  if CheckDrawEndgame(White, Black) then
  begin
    Score := DrawScore;
    goto LabelStore;
  end;
  if CheckDrawEndgame(Black, White) then
  begin
    Score := DrawScore;
    goto LabelStore;
  end;
	//
  if CheckWonEndgame(White, Black, Temp) then
  begin
		if MyColour = White then
    begin
    	Score.MatScore := EndGameScore;
      Score.PosScore := Temp;
      Score.VisitScore := 0;
    end
    else
    begin
    	Score.MatScore := -EndGameScore;
      Score.PosScore := -Temp;
      Score.VisitScore := 0;
    end;
    goto LabelStore;
  end;
  if CheckWonEndgame(Black, White, Temp) then
  begin
		if MyColour = Black then
    begin
    	Score.MatScore := EndGameScore;
      Score.PosScore := Temp;
      Score.VisitScore := 0;
    end
    else
    begin
    	Score.MatScore := -EndGameScore;
      Score.PosScore := -Temp;
      Score.VisitScore := 0;
    end;
    goto LabelStore;
  end;

  //
  for iFile := 1 to 8 do begin
  	OpenFile[White, iFile] := True;
  	OpenFile[Black, iFile] := True;
  end;
  //
  //pawn structure
  //
  PawnPosScore(White, Black, TempPos[White]);
  PawnPosScore(Black, White, TempPos[Black]);
	//
  KnightPosScore(White, Black, TempPos[White]);
  KnightPosScore(Black, White, TempPos[Black]);
  //
  BishopPosScore(White, Black, TempPos[White]);
  BishopPosScore(Black, White, TempPos[Black]);
  //
  RookPosScore(White, Black, TempPos[White]);
  RookPosScore(Black, White, TempPos[Black]);
  //
  QueenPosScore(White, Black, TempPos[White]);
  QueenPosScore(Black, White, TempPos[Black]);
  //
  KingPosScore(White, Black, TempPos[White]);
  KingPosScore(Black, White, TempPos[Black]);
  //
	//asymmetric contributions
  //
  if (GameMode = HumanComputer) or (GameMode = ComputerHuman) then
  begin
  	if GameMode = HumanComputer then
  	begin
    	HumanColour := White;
      ComputerColour := Black;
    end
    else
    begin
    	HumanColour := Black;
      ComputerColour := White;
    end;
    //
  	if (TruePieceCount[ComputerColour, Pawn] < 8) then
    begin
    	if ShowScore then
      	MyPrintf('Computer has less than 8 pawns');
      if (MyColour = ComputerColour) then
      begin
      	Inc(TempPos[MyColour], PawnExchangeScore);
      end
    	else
      begin
      	Inc(TempPos[YourColour], PawnExchangeScore);
      end;
    end;
    //
  	if (TruePieceCount[ComputerColour, Queen] = 0) then
    begin
    	if ShowScore then
      	MyPrintf('Computer has no queen');
      if (MyColour = ComputerColour) then
      begin
      	Inc(TempPos[MyColour], QueenExchangeScore);
      end
    	else
      begin
      	Inc(TempPos[YourColour], QueenExchangeScore);
      end;
    end;
    //
  end;
  Score.PosScore := TempPos[MyColour] - TempPos[YourColour];
  Score.VisitScore := TempVisit[MyColour] - TempVisit[YourColour];
  //
  LabelStore:
  if not Dangerous then
  begin
  	with ScoreCache[MyColour][iCache] do
  	begin
  		ScoreCacheKey := NodeList[Node].NodeFullHashKey;
  		ScoreCacheScore := Score;
  	end;
  end;
end;

initialization

ReadVisit;

end.
