{
ZZZZZZ 5.0, a chess program.
Copyright (C) 1998 G. Wiesenekker
E-mail: gijsbert.wiesenekker@wxs.nl
http://home.wxs.nl/~gijsbert.wiesenekker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}
unit Utils;

{$MODE Delphi}

interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Menus, registry;

const
	AllColumns = -1;

function StrToBool(TheString: AnsiString): Boolean;
function BoolToStr(TheBoolean: Boolean): AnsiString;
function ReadRegValString(Name: AnsiString; DefValue: AnsiString = ''): AnsiString;
procedure WriteRegValString(Name: AnsiString; Value: AnsiString);
procedure UpgradeRegistry(Version: Integer);

procedure AdjustStringGrid(StringGrid: TStringGrid;
	TheColumn: Integer; AdjustHeight: Boolean; TheString: AnsiString);
procedure ClearStringGrid(StringGrid: TStringGrid; ExitIfEmpty: Boolean = True);
procedure StringGridDrawCell(StringGrid: TStringGrid;
	Col, Row: Integer; Rect: TRect; State: TGridDrawState);

implementation

uses Chess;

var RegKey: TRegistry;

function StrToBool(TheString: AnsiString): Boolean;
begin
	if TheString = 'True' then
  	Result := True
  else
  	Result := False;
end;

function BoolToStr(TheBoolean: Boolean): AnsiString;
begin
	if TheBoolean then
  	Result := 'True'
  else
  	Result := 'False';
end;

procedure OpenRegistry;
begin
  RegKey.RootKey := HKEY_LOCAL_MACHINE;
  RegKey.OpenKey(RegKeyZZZZZZ, True);
end;

procedure CloseRegistry;
begin
	RegKey.CloseKey;
end;

function ReadRegValString(Name: AnsiString; DefValue: AnsiString = ''): AnsiString;
begin
	OpenRegistry;
  if (not RegKey.ValueExists(Name)) then
    RegKey.WriteString(Name, DefValue);
  Result := RegKey.ReadString(Name);
  CloseRegistry;
end;

procedure WriteRegValString(Name: AnsiString; Value: AnsiString);
begin
	OpenRegistry;
  RegKey.WriteString(Name, Value);
  CloseRegistry;
end;

procedure UpgradeRegistry(Version: Integer);
var
	RegVersion: Integer;
begin
  RegVersion := StrToInt(ReadRegValString(RegValVersion, '1'));
	if RegVersion < Version then
  begin
  	Application.MessageBox('Registry entry ' + RegKeyZZZZZZ + ' will be upgraded', 'Confirm', MB_OK);
  	RegKey.DeleteKey(RegKeyZZZZZZ);
    WriteRegValString(RegValVersion, IntToStr(Version));
  end;
end;

procedure AdjustStringGrid(StringGrid: TStringGrid;
	TheColumn: Integer; AdjustHeight: Boolean; TheString: AnsiString);
var
	Width , Height: Integer;
  Rect: TRect;
  Col, Row: Integer;
  TotWidth, TotHeight: Integer;
begin
	StringGrid.Canvas.Font := StringGrid.Font;
  Width := StringGrid.Canvas.TextWidth(TheString);
  Height := StringGrid.Canvas.TextHeight(TheString);
  if TheColumn = AllColumns then
  begin
  	for Col := StringGrid.FixedCols to StringGrid.ColCount - 1 do
    	StringGrid.ColWidths[Col] := Width;
  end
  else
  	StringGrid.ColWidths[TheColumn] := Width;
	//Delphi compiler bug: StingGrid.GridWidth not correctly updated
  TotWidth := 0;
  for Col := 0 to StringGrid.ColCount - 1 do
  	TotWidth := TotWidth + StringGrid.ColWidths[Col] + StringGrid.GridLineWidth;
  if StringGrid.ScrollBars = ssVertical then
  	TotWidth := TotWidth + 21
  else
  	TotWidth := TotWidth + 3;
  Rect := StringGrid.BoundsRect;
  Rect.Right := Rect.Left + TotWidth;
  if (AdjustHeight) then
  begin
  	for Row := StringGrid.FixedRows to StringGrid.RowCount - 1 do
  		StringGrid.RowHeights[Row] := Height;
    TotHeight := 0;
    for Row := 0 to StringGrid.RowCount - 1 do
      TotHeight := TotHeight + StringGrid.RowHeights[Row] + StringGrid.GridLineWidth;
    TotHeight := TotHeight + 3;
	  Rect.Bottom := Rect.Top + TotHeight;
  end;
  StringGrid.BoundsRect := Rect;
end;

procedure ClearStringGrid(StringGrid: TStringGrid; ExitIfEmpty: Boolean = True);
var
	Col, Row: Integer;
label return;
begin
	for Row := StringGrid.FixedRows to StringGrid.RowCount - 1 do
  begin
  	for Col := StringGrid.FixedCols to StringGrid.ColCount - 1 do
    begin
    	if ExitIfEmpty and (StringGrid.Cells[Col, Row] = '') then
				Exit;
      StringGrid.Cells[Col, Row] := '';
    end;
  end;
end;

procedure StringGridDrawCell(StringGrid: TStringGrid;
	Col, Row: Integer; Rect: TRect; State: TGridDrawState);
var
	Org, Map: AnsiString;
  WhiteSquare: Boolean;
  WhitePiece: Boolean;
begin
	StringGrid.Canvas.Font := StringGrid.Font;
  Org := StringGrid.Cells[Col, Row][1];
  WhitePiece := False;
  if (Org = wP) then
  begin
  	WhitePiece := True;
    Map := 'o'
  end
  else if (Org = wN) then
  begin
  	WhitePiece := True;
    Map := 'm'
  end
  else if (Org = wB) then
  begin
  	WhitePiece := True;
    Map := 'v'
  end
  else if (Org = wR) then
  begin
  	WhitePiece := True;
    Map := 't'
  end
  else if (Org = wQ) then
  begin
  	WhitePiece := True;
    Map := 'w'
  end
  else if (Org = wK) then
  begin
  	WhitePiece := True;
    Map := 'l'
  end
  else if (Org = bP) then
  begin
    Map := 'o'
  end
  else if (Org = bN) then
  begin
    Map := 'm'
  end
  else if (Org = bB) then
  begin
    Map := 'v'
  end
  else if (Org = bR) then
  begin
    Map := 't'
  end
  else if (Org = bQ) then
  begin
    Map := 'w'
  end
  else if (Org = bK) then
  begin
    Map := 'l'
  end
  else if (Org = nE) then
  begin
    Map := ' '
  end
  else
    raise EChessError.Create('StringGrid out of range');
  WhiteSquare := not odd (Col + Row);
  if WhiteSquare then
		StringGrid.Canvas.Brush.Color := WhiteBackGroundColor
  else
		StringGrid.Canvas.Brush.Color := BlackBackGroundColor;
  StringGrid.Canvas.FillRect(Rect);
	if WhitePiece then
  	StringGrid.Canvas.Font.Color := clWhite
  else
  	StringGrid.Canvas.Font.Color := clBlack;
  StringGrid.Canvas.TextRect(Rect, Rect.Left, Rect.Top, Map);
end;

initialization

RegKey := TRegistry.Create;

finalization

RegKey.Destroy;

end.
